/*
 Arduino temperature and relative humidty (DHT11) datalogger using SD card. 
 
 Each time the Arduino is reset/powered up, a new data log file is created with a
 unique filename, leaving all other data log files intact on the SD card. To create the
 unique filename, no extra hardware such as a real time clock or other connection
 to the outside world is needed.
 
 The code to create each unique filename does not use C language pointers, keeping
 code easy to understand and adapt for other SD-related uses. The code also avoids using
 any type of random number as the basis for the filename. All files on the SD card are
 sequence numbered and thus are chronological.
 
 Any suitable SD card can be used - no "initialisation" of a formatted SD card is necessary.
 
 The hardware:
 Arduino + Ethernet shield with SD card slot
 DHT11 temperature and relative humidity sensor and 4k7 res 
      (resistor connected between DHT11 pins 1 (Vcc) and 2 (Data)
 Green and red LEDs and 2x330R res
 
 The circuit:
 * SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4
 * Ethernet shield as follows:
 ** W5100 CS - pin 10
 
 * DHT11 data pin - pin 5
 * Green LED - pin 8 // flashes when data read and written to SD card - approx each minute
 * Red LED - pin 7   // flashes each second approx to show the data logger is running
 SD card file is closed when green LED off - can safely remove power or card when green LED not on

 Sketch ID: trhloggr.ino 5 Jan 13 by Harold Corbould
 Uses SD card file creation example by David A. Mellis created Nov 2010
 Modified 9 Apr 2012 by Tom Igoe
 This example code is in the public domain.
  */
  
#include <TimerOne.h>
#include <dht11.h>
#include <SD.h>

File NumberPriorFile;
File DataFile;
unsigned long filenumprior = 10000;
unsigned long filenumnew = 10001;  // set new number as same in case no previous creation of fnlog.txt file
byte abyte;
char NumberPrior[6] = "10000"; // set up for 10001-99999 (89,999 values)
//Next string array eventually holds new SD file name including unique nnnnn number
char NumberNew[13] = "10001"; // set up for 10001-99999 (89,999 values)
int DHTread = 0;
int inSwitch = 12;   // activate serial communication with PC via USB cable
int NumberPriorPtr = 0;
dht11 DHT11;   // name the temp and RH sensor
#define DHT11PIN 5  // pin 5 PWM is data input pin from DHT11
#define greenLEDPIN 8  // pin 8 High Low
#define redLEDPIN 7  // pin 7 High Low

void setup()
{
  //  pinMode(inSwitch,INPUT_PULLUP); // set pin 'inSwitch' to use internal pull-up resistor
  pinMode(greenLEDPIN,OUTPUT); // set pin mode
  pinMode(redLEDPIN,OUTPUT); // set pin mode

  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  delay(1000);
  while (!Serial) {
    ; // insert delay here if need to wait for serial port to connect
  }
  delay(1000); // do nothing for 1s

  Serial.println("Initializing SD card...");
  // On the Ethernet Shield, CS is pin 4. It is set as an output by default.
  // Note that even if it is not used as the CS pin, the hardware SS pin
  // (10 on most Arduino boards, 53 on the Mega) must be left as an output
  // or the SD library functions will not work.
  pinMode(10, OUTPUT);
  if (!SD.begin(4)) {
    Serial.println("          Initialization failed!");
    return;
  }
  Serial.println("  Initialization successful.");
/*
  Use interrupt approach for short intervals, otherwise use delay(ms) in main loop as needed
  8s between data recording too frequent for temp and RH
  Timer1.initialize(8000000);  // max period possible is ~8.3s or ~0.12Hz (1,000,000us = 1Hz)
  Timer1.attachInterrupt(update); // when timer overruns, call the 'update' procedure below
*/
  digitalWrite(greenLEDPIN,LOW); // off at start
  digitalWrite(redLEDPIN,LOW); // off at start
  delay(2000); // delay 2s - increase delay if necessary

  DHTread = DHT11.read(DHT11PIN); // read data from DHT11 sensor
  Serial.print("DHTread = ");
  Serial.println(DHTread);
  switch (DHTread) // depending on the error code return, do one of the following:
  {
  case DHTLIB_OK: 
    // print OK result to SD card
    Serial.println("DHT11 setup OK");
    break;
  case DHTLIB_ERROR_CHECKSUM: 
    // print DHTLIB_ERROR_CHECKSUM result to SD card
    // send DHTLIB_ERROR_CHECKSUM msg to serial port
    break;
  case DHTLIB_ERROR_TIMEOUT: 
    // print DHTLIB_ERROR_TIMEOUT result to SD card
    // send DHTLIB_ERROR_TIMEOUT msg to serial port
    break;
  case DHTLIB_ERROR_ACKNOW: 
    // print DHTLIB_ERROR_ACKNOW result to SD card
    // send DHTLIB_ERROR_ACKNOW msg to serial port
    break;
  default: 
    // print DHTLIB_ERROR_UNKNOWN result to SD card
    // send DHTLIB_ERROR_UNKNOWN msg to serial port
    break;
  }

  /*   
   ===================
   Determine if a file fnlog.txt exists
   If exists, then read the last number nnnnn
   If not exists, then skip as variable filenumprior already set as 10000
   Note: file fnlog.txt file should never be removed from the SD card
   unless all other data log files are also removed, otherwise cannot
   guarantee filename uniqueness
   ===================
   */
  if (!SD.exists("fnlog.txt")) {  // not exists, send serial msg
    Serial.println("File fnlog.txt does not exist; file will be created later in setup.");
  } 
  else {  // read fnlog.txt
    Serial.println("Opening fnlog.txt file ...");
    // open the file and read each byte ...
    NumberPriorFile = SD.open("fnlog.txt", FILE_READ);
    // read from the file until there's nothing else in it:
    while (NumberPriorFile.available()) {
      abyte =(NumberPriorFile.read());
      NumberPrior[NumberPriorPtr] = abyte; // store the byte char regardless of 0 or otherwise
      if (abyte != 10 ){  // not end of 0Dh 0Ah
        ++NumberPriorPtr; // bump the pointer ready for next read
      } 
      else {
        NumberPrior[NumberPriorPtr-1] = 0; // place 00h at end of five numeral chars to terminate the string with 00h
        NumberPriorPtr = 0; // reset the pointer for next line and ovwerwrite any previous number value
      } // endif abyte = 10
    } // no more number lines in file
    Serial.println("Last line read ...");
    Serial.print("NumberPrior = ");
    Serial.println(NumberPrior);
    filenumnew = (NumberPrior[0]-48)*10000+(NumberPrior[1]-48)*1000+(NumberPrior[2]-48)*100+(NumberPrior[3]-48)*10+(NumberPrior[4]-48);
    Serial.print("filenumOLD = ");
    Serial.println(filenumnew);  // but it's still the old number at this point
    ++filenumnew;
    if (filenumnew > 99999){
      return; // run out of 5 digit numbers! (could make filenumnew = 10001 - hardly likely 89,999 files on the same SD card) 
    }         
    NumberNew[4] = 48 + (filenumnew % 10);
    NumberNew[3] = 48 + ((filenumnew % 100) - (filenumnew % 10))/10;  
    NumberNew[2] = 48 + ((filenumnew % 1000) - (filenumnew % 100))/100;  
    NumberNew[1] = 48 + ((filenumnew % 10000) - (filenumnew % 1000))/1000;  
    NumberNew[0] = 48 + ((filenumnew - (filenumnew % 10000))/10000);
    NumberNew[5] = 0;  // terminator of five digit number as string
    // close the fnlog.txt file
    NumberPriorFile.close(); // finished reading, close so can open for writing
    Serial.println("Closing fnlog.txt ...");
  } // end of file access if fnlog.txt already existed

    // from here, fnlog.txt is reopened if it aleady existed, or is created if did not exist
    // either way, the "new " file number is written to fnlog.txt
    // now write the newest number to the fnlog.txt
    Serial.println("Re-opening fnlog.txt to write ..."); 
    NumberPriorFile = SD.open("fnlog.txt", FILE_WRITE);  // re-open fnlog.txt and record latest number used
    Serial.print("Writing filenumNEW = ");
    Serial.println(filenumnew);  
    NumberPriorFile.println(NumberNew);
    // re-close the file after adding new number to the records:
    Serial.println("Re-closing fnlog.txt ...");
    NumberPriorFile.close();
  
  // Now expand the NumberNew string - add "log" and ".txt" to create the full unique filename 
  // for the data log
  // shift number content ASCII chars three places to right
  for (int i = 4; i > -1; i--){
    NumberNew[i+3] =  NumberNew[i];
  }
  NumberNew[0] = 108; // l
  NumberNew[1] = 111; // o
  NumberNew[2] = 103; // g
  NumberNew[8] = 46; // dot/period/full stop
  NumberNew[9] = 116; // t
  NumberNew[10] = 120; // x
  NumberNew[11] = 116; // t
  Serial.print("New full filename for data log: " );
  Serial.println(NumberNew);
  /*
  Anytime after this point, NumberNew may be used as a basis for a new data log file 
   with a name which will be unique, leaving all previously created data log files
   on the SD card intact. 
   */
  } // end of setup

void update() { // this interrupt-triggered (or called by main loop) procedure is the main data gathering process
  // flashes green led each data read
  digitalWrite(greenLEDPIN,HIGH); // on 
  // Open the data log file and write dummy data...
  delay(100);
  DHTread = DHT11.read(DHT11PIN); // read new data from DHT11
  DataFile = SD.open(NumberNew, FILE_WRITE);
  // if the file successfully opened, write T and RH data to file:
  if (DataFile) {
    DataFile.print(DHT11.temperature);
    DataFile.print("," );
    DataFile.println(DHT11.humidity);
    DataFile.close();  // close the file:
    // serial output for initial testing ...
    Serial.print(DHT11.temperature);
    Serial.print("  ");
    Serial.print(DHT11.humidity);
    Serial.print("  ");
  }
  digitalWrite(greenLEDPIN,LOW); // off 
}

void loop()
{
  // use nested delays to reduce SD card data storage needs, while producing interim feedback
  // values below create data log each minute approx
  update();
  for (int i = 0; i < 60 ; i++){  // 60loops * 1000ms/loop = ~1min per reading
    digitalWrite(redLEDPIN,HIGH); // LED on
    delay(100);  // on
    digitalWrite(redLEDPIN,LOW); // LED off
    delay(900); // change to 59900 for data logging each 10 mins approx
    Serial.print(".");
  } // end i loop
  Serial.println(".");

} // end main loop

